package decac

import scala.collection.mutable.Set
import scala.collection.mutable.HashSet
import java.lang.Math
import jllvm.LLVMType
import jllvm.LLVMIntegerType

class EnumeratedType(m: Module,n: String,p: Option[EnumeratedType],syms: List[String]) extends ActualType(n,Some(m)) {
  val parent: Option[EnumeratedType] = p
  val symbols: List[String] = syms
  var values: List[Int] = Nil
  var childEnums: Set[EnumeratedType] = new HashSet[EnumeratedType]()
  
  def typeConstant: Boolean = true
  
  protected def assign_symbol_values(s: Int): Int = {
    values = Nil
    var start = s
    var i = 0
    for(symbol <- symbols) {
      values = values ::: List(i)
      i += 1
    }
    start += values.length
    for(child <- childEnums)
      start = child.assign_symbol_values(start)
    return start
  }
  
  def assign_values(): Unit = parent match {
    case None => assign_symbol_values(0)
    case Some(parent) => parent.assign_values
  }
  
  protected def calculateRepresentationSize: Int = {
    values.length + childEnums.map(child => child.calculateRepresentationSize).foldLeft(0)((x,y) => x + y)
  }
  
  protected def representationSize: Int = parent match {
    case None => calculateRepresentationSize
    case Some(parent) => parent.representationSize
  }
  
  override def compile(substitution: Substitution): LLVMType = {
    val numBits = representationSize
    new LLVMIntegerType(numBits)
  }
}
