package decac

import scala.collection.mutable.Map
import scala.collection.mutable.HashMap

trait Scopeable {
  def name(): String
  def scope(): Scope
}

class Scope(p: Scope) {
  protected var symbols: Map[String,Scopeable] = new HashMap[String,Scopeable]()
  
  def lookup(name: String): Scopeable = {
    val result = symbols.get(name)
    if(result.isDefined)
      result.get
    else
      if(parent != null) parent.lookup(name) else throw new UndeclaredIdentifierException(name)
  }
  
  def declare(obj: Scopeable) = {
    symbols.put(obj.name,obj)
  }
  
  val parent: Scope = p
  
  def encloses(s: Scope): Boolean = {
    var next = s
    while(next != null) {
      if(next == this)
        return true
      next = next.parent
    }
    return false
  }
}

abstract class Binding(n: String,s: Scope) {
  val name = n
  val scope = s
  
  val correctType: Boolean
  
  def evaluate: Scopeable = scope.lookup(name)
}
